from distribute_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages, findall
from setuptools.extension import Extension
from Cython.Build import cythonize
import numpy as np

# extensions = [
#     Extension(
#         "vg/time_series_analysis/cresample",
#         ["vg/time_series_analysis/cresample.pyx"],
#         include_dirs=np.get_include()),
#     Extension(
#         "vg/ctimes",
#         ["vg/ctimes.pyx"],
#         include_dirs=np.get_include()),
# ]

extensions = [Extension("vg/*/*", ["vg/*/*.pyx"],
                        include_dirs=np.get_include())]

setup(
    name="vg",
    version="1.3a1",
    packages=["vg", "vg.core", "vg.meteo", "vg.time_series_analysis"],
    ext_modules=cythonize(extensions),
    scripts=["distribute_setup.py"],

    install_requires=[
        'matplotlib',
        'scipy',
        'numpy',
        'pandas',
        # 'cython',
        'tqdm',
        ],
    # dependency_links=[
    #     'http://sourceforge.net/projects/matplotlib/files/matplotlib/matplotlib-1.1.1/',
    #     'http://sourceforge.net/projects/scipy/files/scipy/0.11.0b1/',
    #     'http://sourceforge.net/projects/numpy/files/NumPy/1.6.2/',
    #     ],

    package_data={
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.dat', '*.met', "*.rst", "*.pyx"],
        # 'doc': ['*.html', '*.rst'],
        },

    include_package_data=True,

    # metadata for upload to PyPI
    author="Dirk Schlabing",
    author_email="dirk.schlabing@iws.uni-stuttgart.de",
    description="A Vector Autoregressive Moving Average Weather Generator",
    license="BSD",
    keywords=("weather generator vector autoregressive moving average " +
        "time series analysis"),
    #url = "http://example.com/HelloWorld/",   # project home page, if any

    # could also include long_description, download_url, classifiers, etc.
    long_description=\
        """The weather generator VG is a single-site Vector-Autoregressive
(Moving Average) weather generator that was developed for hydrodynamic and
ecologic modelling of lakes. It includes a number of possibilities to
define scenarios."""
)
