import Test3DStatHelpers
import numpy as np
import matplotlib.pyplot as mp
from matplotlib import cm

def emp_density_copula_plot(X, Y, NumberBins=15, Radius=0.1, ax=None):
    """Taken from Claus and Marias codes. Simplified to plot just the
    corrected empirical copula density."""

    # Initialisationen
    GridBoundaries = []
    BinWidth = (1. / NumberBins)
    AnzahlPunkte = np.zeros([NumberBins, NumberBins])
    AnzahlPunkteProSpalte = np.zeros(NumberBins)
    AnzahlPunkteProZeile = np.zeros(NumberBins)
    MidBinsXDir = np.zeros([NumberBins, NumberBins])
    MidBinsYDir = np.zeros([NumberBins, NumberBins])

    for i in range(0, NumberBins + 1, 1):
        GridBoundaries.append(i * BinWidth)
    # Baue Anzahl Punkte in Kaestle zusammen (AnzahlPunkte)
    for i in range(0, len(X)):
        # schaue in jedes Bin in ii-Richtung
        for j in range(0, (len(GridBoundaries) - 1)):
            bot_ii = GridBoundaries[j]
            top_ii = bot_ii + BinWidth
            mid_ii = (bot_ii + top_ii) / 2.
            # schaue in jedes Bin in jj-Richtung
            if (bot_ii <= X[i]) and (top_ii > X[i]):
                # Anzahl_ii[j] = Anzahl_ii[j] + 1
                AnzahlPunkteProSpalte[j] += 1
                for k in range(0, (len(GridBoundaries) - 1)):
                    bot_jj = GridBoundaries[k]
                    top_jj = bot_jj + BinWidth
                    mid_jj = (bot_jj + top_jj) / 2.
                    # wenn ii-Bin und jj-Bin bekannt ist, dann erhoehe Anzahl 
                    # in diesem Bin um 1
                    if (bot_jj <= Y[i]) and (top_jj > Y[i]):
                        # Anzahl_jj[k] = Anzahl_jj[k] + 1
                        AnzahlPunkteProZeile[k] += 1
                        AnzahlPunkte[j, k] += 1

    # Bastle Array fuer
    # x-coordinaten (MidBinsXDir)
    # y-coordinaten (MidBinsYDir)
    # der "Kaestle"
    for j in range(0, (len(GridBoundaries) - 1)):
        bot_ii = GridBoundaries[j]
        top_ii = bot_ii + BinWidth
        mid_ii = (bot_ii + top_ii) / 2.
        for k in range(0, (len(GridBoundaries) - 1)):
            bot_jj = GridBoundaries[k]
            top_jj = bot_jj + BinWidth
            mid_jj = (bot_jj + top_jj) / 2.
            MidBinsXDir[j, k] = mid_ii
            MidBinsYDir[j, k] = mid_jj

    fs = np.shape(MidBinsXDir)[0] * np.shape(MidBinsXDir)[1]
    RasterX = MidBinsXDir.reshape(fs)
    RasterY = MidBinsYDir.reshape(fs)

    # Da ist das Problem!
    # todo: if array too big then don't do kernel smoothing
    try:
        KernelSmoothedZVals = \
        Test3DStatHelpers.KernelSmoothing(RasterX, RasterY, X, Y, Radius)

        KernelSmoothedZValsRS = KernelSmoothedZVals.reshape(MidBinsXDir.shape)
        AnzahlPunkteProZeile = KernelSmoothedZValsRS.sum(axis=0)
        AnzahlPunkteProSpalte = KernelSmoothedZValsRS.sum(axis=1)

        RasterX = RasterX.reshape(MidBinsXDir.shape)
        RasterY = RasterY.reshape(MidBinsXDir.shape)

        # ------------------------------------------------------------
        #                 Integrieren, Standardisieren, Differenzieren
        # ------------------------------------------------------------

        #Integrieren
        Copula = Test3DStatHelpers.MatrixIntegrieren(KernelSmoothedZValsRS)

        #Standardisieren
        RechteDimension = KernelSmoothedZValsRS.shape[1] - 1
        scaleFaktor = Copula[0][RechteDimension]
        scaledCopula = (Copula / float(scaleFaktor))

        #Differenzieren
        CopulaDensity = Test3DStatHelpers.MatrixDifferenzieren(scaledCopula)

        # divided by the *area* of a discretized square of the original raw
        # data
        length = 1.0 / KernelSmoothedZValsRS.shape[1]
        #print "length >> \n", length
        area = length ** 2
        CorrectedCopulaDensity = CopulaDensity / area

        if ax is None:
            con_func = mp.contourf
        else:
            con_func = ax.contourf
        con_func(RasterX, RasterY, CorrectedCopulaDensity, 8, cmap=cm.jet)
        if ax is None:
            mp.colorbar(format='%.4f') # draw colorbar
    except MemoryError:
        print "KernelSmoothing Array is too big!"
        print "Printing aborted!"
