
# encoding: utf-8
"""
ParameterFreeShit.py

Created by Claus Haslauer
University of Stuttgart
Department of Hydrology and Geohydrology

Development started on 2010-02-10.
Copyright (c) 2010. All rights reserved.

v 0.1:
This is a program for non-parametric estimation of
    - an empirical distribution function (EDF) if a univariate data-set is given
    - a regression of y based on x if a bivariate data-set (x,y) is given

- It is optional to optimize the kernel width using ML based "leave-one-out" optimization
- Three kernel-shapes are implemented: Triangular, Epanochanikazoniakovolic, and Gaussian
"""

# TODO think about what to do at "edges"
# TODO check how big data-sets can be
# TODO solver sometimes behaves funny: sometimes fmin workes, sometimes fmin_bfgs

# :/Users/claushaslauer/Documents/_CodeDev/

import numpy
import math
import matplotlib.pyplot as plt
import scipy.optimize as sco

def main():
    # ------------------------------------------------------------
    #                                             define variables
    # ------------------------------------------------------------

    # path and filename where the data to be regressed is
        # 1 dat-set for parameter-free distribution function
        # 2 data-sets for parameter-free regression
    path = "/Users/claushaslauer/Desktop/present work/ParameterFreeShit/"
    filename = "Drawdown.dat" #"ElevationPrecipitation.txt" #"Galton.txt"

    # number of discretization steps
    nDiscrSt = 100

    # kinds (shapes of kernels) can be
    #   Tri   Triangular
    #   Epa   Epanemichowuiertnljkrhgtf
    #   Gau   GuessWhat
    kernelKind = 'Gau'

    # things related to Kernel Width -----------------------
    optimizeKernelWidth = True
    #   if optimizeKernelWidth=True then this is a starting value
    #   otherwise this is the kernelWidth to be used
    kernelWidthStart = 0.06

    # ------------------------------------------------------------
    #                                           start real program
    # ------------------------------------------------------------
    # read data
        # if 1 colum: EDF-construction
        # if 2 columns: regression
    data = numpy.loadtxt(path + filename, comments='#')

    print "shape of data: ", numpy.shape(data)[1]

    # do parameter free EDF --------------------------------
    if numpy.shape(data)[1] == 1:

        print "done reading data!"
        print 'doing parameter-free EDF'
        xVect = data   # *24*36

        discr = calcDiscr(xVect, nDiscrSt)

        # optimize kernel width:
        if optimizeKernelWidth == True:
            kernelWidth = calcOptKernelWidth(kernelWidthStart, xVect, kernelKind)

        # calculate the kernel Matrix
        kernelMatrix = kernelCalc(xVect, discr, kernelWidth, kernelKind) #stepSize*4.0

        # the big part is done, cleanup now
        densFunc = numpy.sum(kernelMatrix, axis=0) / float(numpy.shape(xVect)[0])
        print "shape density function: ", numpy.shape(densFunc)

        # print density
        printOneFunc(discr, densFunc)

        # print distributions
        empCdf = edfCalc(xVect)
        kernelCdf = numpy.cumsum(densFunc) / numpy.sum(densFunc)
        printTwoFunc(xVect, empCdf, discr, kernelCdf)
        print "Done!"

    # do parameter free REGRESSION -------------------------
    if numpy.shape(data)[1] == 2:
        print "done reading data!"
        print 'doing parameter-free Regression'

        xVect = data[:, 0]
        yVect = data[:, 1]

        discr = calcDiscr(xVect, nDiscrSt)

        # optimize kernel width:
        if optimizeKernelWidth == True:
            kernelWidth = calcOptKernelWidth(kernelWidthStart, xVect, kernelKind)

        print "Done optimizing kernel width"

        # calculate the kernel Matrix
        if optimizeKernelWidth == False:
            kernelWidth = kernelWidthStart

        kernelMatrix = kernelCalc(xVect, discr, kernelWidth, kernelKind) #stepSize*4.0
        print "Done calculating Kernel Matrix."

        tempSum = numpy.sum(kernelMatrix, axis=0)

        temp = yVect[:, numpy.newaxis] * kernelMatrix
        tempSp = numpy.sum(temp, axis=0)

        regrFunc = tempSp / tempSum
        #printOneFunc(discr, regrFunc)

        print "start printing figures:"
        printTwoFunc(xVect, yVect, discr, regrFunc)

        print "Done!"

def calcOptKernelWidth(kernelWidthStart, xVect, kernelKind):
    p = [kernelWidthStart]

    print "---- Liste p wird der Optimierung uebergeben:"
    print type(p)
    print type(p[0])

    print "---- start Optimierung:"
    opt = sco.fmin(MLM, p, args=(xVect, kernelKind), full_output=1)
    optiParas = opt[0]
    optKernel = optiParas[0]
    print "optKernelWidth: ", optKernel
    kernelWidth = optKernel

    return kernelWidth

def MLM(p, x, kernelKind):
    """docstring for MLM"""
    print "-- now inside Function MLM"
    kernelWidth = float(p[0])

    print "KernelWidth: ", kernelWidth

    xVect = x
    #print "xVect: ", xVect
    # calculate kernel Matrix with measurement Values as "xVect" and as "discretization"
    crossValMatrix = kernelCalc(xVect, xVect, kernelWidth, kernelKind)
    crossValVec = numpy.sum(crossValMatrix, axis=0) - numpy.diagonal(crossValMatrix)
    lnCrossVal = numpy.log(crossValVec)
    optSum = -1 * numpy.sum(lnCrossVal)
    return optSum


def calcDiscr(xVect, nDiscrSt):
    # determine range --------------------------------------
    #    a little to the left of max(x)
    #    so that they are nice numbers
    range_min = numpy.round(numpy.min(xVect) - 1.2, 3)
    range_max = numpy.round(numpy.max(xVect) + 1.2, 3)
    stepSize = (range_max - range_min) / nDiscrSt
    discr = numpy.arange(range_min, range_max, stepSize)
    return discr


def edfCalc(xVals):
    """docstring for edfCalc"""
    ErwTr = 1. / (2. * len(xVals))
    EDFWert = 1. / (len(xVals))
    asort = numpy.arange((ErwTr), (1.), (EDFWert))
    ind = numpy.argsort(numpy.argsort(xVals))
    # ind2 = len(ind)-ind-1
    StdF1 = asort[ind]
    return StdF1


def printOneFunc(xVals, yVals):
    """docstring for printDensFunc"""
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.plot(xVals, yVals, "*-")
    plt.show()


def printTwoFunc(xVals1, yVals1, xVals2, yVals2):
    """docstring for printDensFunc"""
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.plot(xVals1, yVals1, "*", color='red')
    ax.plot(xVals2, yVals2, "-", color='blue')
    plt.show()


def kernelCalc(xVect, discretization, kernelWidth, kind):
    """
    >>> import numpy
    >>> numpy.array([1299, 1488, 955, 1340, 894, 2600, 1721])
    >>> kernelWidth = 200.0
    >>> kind = 'Tri'
    >>> kernelCalc(xVect, xVect, kernelWidth, kind)
    >>> [[ 0.005     0.000275  0.        0.003975  0.        0.        0.      ]
         [ 0.000275  0.005     0.        0.0013    0.        0.        0.      ]
         [ 0.        0.        0.005     0.        0.003475  0.        0.      ]
         [ 0.003975  0.0013    0.        0.005     0.        0.        0.      ]
         [ 0.        0.        0.003475  0.        0.005     0.        0.      ]
         [ 0.        0.        0.        0.        0.        0.005     0.      ]
         [ 0.        0.        0.        0.        0.        0.        0.005   ]]
    """
    #print "--now inside kernelCalc--" 
    nDiscr = numpy.shape(discretization)[0]
    nXVect = numpy.shape(xVect)[0]

    lefty = numpy.outer(numpy.ones((nXVect)), discretization)
    # print "shape lefty: ", numpy.shape(lefty)

    diOnes = numpy.ones((nDiscr, nXVect))
    righty = (xVect * diOnes).T
    # print "shape righty: ", numpy.shape(righty)

    # das ist Difference Diskretisierung (x) minus Messwerte (X)
    # also "x-X"
    calcDifMeasObs = lefty - righty  #  ( (diOnes*discretization)-(diOnes*xVect).T ) * -1

    if kind == 'Tri':
        # das ist der Dreieckskernel
        residuals = 1 - abs(calcDifMeasObs) / kernelWidth
        # das berechnet die Kernelmatrix
        kernelMatrix = numpy.where(residuals > 0.0, residuals / kernelWidth, 0.0)
    elif kind == 'Epa':
        residuals = 1 - abs(calcDifMeasObs) ** 2 / kernelWidth ** 2
        temp = numpy.where(residuals > 0.0, residuals, 0.0)
        kernelMatrix = temp / kernelWidth
    elif kind == 'Gau':
        # print "kernelWidth: ", type(kernelWidth)
        preTerm = 1.0 / (math.sqrt(2 * math.pi) * kernelWidth)
        # print "preTerm: ", type(preTerm)
        # print "calcDifMeasObs", type(calcDifMeasObs)
        kernelMatrix = preTerm / numpy.exp(calcDifMeasObs ** 2 / (2 * kernelWidth ** 2))
    else:
        "You chose a wrong or no kernel. Damn!"


    return kernelMatrix


def read_file(filename):
    f = open(filename, 'r')
    outarray = []
    for line in f.readlines():
        split = line.strip().split()
        outarray.append([ float(item) for item in split ])
    print "done reading file ", filename
    f.close()
    return numpy.array(outarray).squeeze()



if __name__ == '__main__':
    import vg
    times, met = vg.read_met()
    theta = met["theta"]
    #memory error evasion
    theta = theta[:366 * 24]
    kernelWidth = calcOptKernelWidth(.01, theta, "Gau")
    discr = calcDiscr(theta, 20)
    # calculate the kernel Matrix
    kernelMatrix = kernelCalc(theta, discr, kernelWidth, "Gau") #stepSize*4.0

    # the big part is done, cleanup now
    densFunc = numpy.sum(kernelMatrix, axis=0) / float(numpy.shape(theta)[0])
    print "shape density function: ", numpy.shape(densFunc)

    # print density
    printOneFunc(discr, densFunc)

    # print distributions
    empCdf = edfCalc(theta)
    kernelCdf = numpy.cumsum(densFunc) / numpy.sum(densFunc)
    printTwoFunc(theta, empCdf, discr, kernelCdf)


#    main()
