import numpy as np
from numpy.testing import assert_almost_equal, TestCase, run_module_suite
import vg.time_series_analysis.distributions as ds

distributions = dict([(name, obj) for name, obj in ds.__dict__.items()
                      if isinstance(obj, ds.MyDist)])
if "SkewNormal" in distributions and not ds.owens:
    del distributions["SkewNormal"]
dists_frozen = dict([(name, dist(*dist._feasible_start))
                     for name, dist in distributions.items()])
quantiles = np.linspace(0, 1, 11)
quantiles_more = np.linspace(0.001, .999, 500)


class Test(TestCase):
    def test_fit(self):
        """Is fit able to reproduce parameters of a ppf-generated sample?"""
        print
        for dist_name, dist in distributions.items():
            print "\t", dist_name
            known_params = dist._feasible_start
            x = dist.ppf(quantiles_more, *known_params)
            if hasattr(dist, "_additional_kwds"):
                kwds = dist._additional_kwds
            else:
                kwds = {}
            fitted_params = dist.fit(x, **kwds)
            if len(kwds) > 0:
                known_params = known_params[:-len(kwds)]
            assert_almost_equal(known_params, fitted_params, decimal=1)

    def test_roundtrip(self):
        """Is the ppf really the inverse of the cdf?"""
        print
        for dist_name, dist_frozen in dists_frozen.items():
            print "\t", dist_name
            x = dist_frozen.ppf(quantiles)
            quantiles_back = dist_frozen.cdf(x)
            assert_almost_equal(quantiles, quantiles_back, decimal=5)

    def test_scalar(self):
        """Do pdf, cdf and ppf accept and return scalar values?"""
        print
        for dist_name, dist_frozen in dists_frozen.items():
            print "\t", dist_name
            self.assert_(np.isscalar(dist_frozen.pdf(1)))
            self.assert_(np.isscalar(dist_frozen.cdf(1)))
            self.assert_(np.isscalar(dist_frozen.ppf(.5)))

if __name__ == "__main__":
    run_module_suite()
